@@ -14,12 +14,6 @@
 #include <utility>
 #include <cmath>
 
-#ifdef _WIN32
-#define ALIGN_8 alignas(8)
-#else
-#define ALIGN_8 __attribute__ ((aligned (8)))
-#endif
-
 class DataSet;
 
 typedef struct
@@ -28,7 +22,7 @@ typedef struct
     int length;
     int capacity;
 
-    char values[8] ALIGN_8;
+    alignas(8) char values[8];
 
 } Block;
 
